/*
    Tic-Tac-Toe 68k v1.0
    Copyright (C) 2001 John David Ratliff
    Questions or Comments: 
    	E-Mail: jdratlif@cs.indiana.edu
    	URL:	http://www.technoplaza.net 
    	
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 7/4/01; 11:00:51 AM

#include "ttt.h"

#include <stdlib.h>

enum Bool {FALSE,TRUE};

// check for a winning move by player
short int oneToWin(SQUARE *board, short int turn) {
	short int loop, move = -1, p1, p2, p3;
	
	for (loop = 0; loop < WIN_COMBOS; loop++) {
		p1 = winnings[loop][0];
		p2 = winnings[loop][1];
		p3 = winnings[loop][2];
		
		if (board[p1].held == turn && board[p2].held == turn && board[p3].held == EMPTY) {
			move = p3;
		} else if (board[p1].held == turn && board[p3].held == turn && board[p2].held == EMPTY) {
			move = p2;
		} else if (board[p2].held == turn && board[p3].held == turn && board[p1].held == EMPTY) {
			move = p1;
		}
	}
	
	return move;
}

// checks to see if a winning combination is still winnable (for the AI)
short int isWinnable(SQUARE *board, short int combo) {
	short int h1, h2, h3, winnable = FALSE;
	
	h1 = board[winnings[combo][0]].held;
	h2 = board[winnings[combo][1]].held;
	h3 = board[winnings[combo][2]].held;
	
	if (h1 != X && h2 != X && h3 != X) {
		winnable = TRUE;
	}
	
	return winnable;
}

// picks a combination to play - returns -1 if no winning combos exist
short int findCombo(SQUARE *board) {
	short int randNum, combo = -1, count = 0;
	
	do {
		randNum = rand() % WIN_COMBOS;
		
		if (isWinnable(board,randNum)) {
			combo = randNum;
		}
	} while (combo == -1 && ++count < MAX_TRY);
	
	return combo;
}

// picks the move to play in the winning combo the AI has selected
short int findMoveInCombo(SQUARE *board, short int combo) {
	short int loop, move = -1, pos;
	
	for (loop = 0; loop < COMBO_SIZE; loop++) {
		pos = winnings[combo][loop];
		
		if (board[pos].held == EMPTY) {
			move = pos;
		}
	}
	
	return move;
}

// if no winning moves exist, find the first open square
short int findOpenMove(SQUARE *board) {
	short int loop, move = -1;
	
	for (loop = 0; loop < TOTAL_SQUARES; loop++) {
		if (board[loop].held == EMPTY) {
			move = loop;
		}
	}
	
	return move;
}

// plays the AI's position defensively
short int aiPlay(SQUARE *board, short int combo) {
	short int turn = !X, move;

	if ((move = oneToWin(board,turn)) != -1) {
		// always play winning move first
		play(board,turn,move);
	} else if ((move = oneToWin(board,!turn)) != -1) {
		// if we can't win, don't let them win either
		play(board,turn,move);
	} else {
		// standard strategy move -- play to win, where possible
		
		// if we can't win with our current combo, find a new one
		if (!isWinnable(board,combo)) {
			combo = findCombo(board);
		}
		
		if (combo == -1) {
			// if no winning moves exist, pick a random move
			move = findOpenMove(board);
		} else {
			// otherwise, play to win
			move = findMoveInCombo(board,combo);
		}
		
		// play our selected move
		play(board,turn,move);
	}

	// the returned combo should never be -1, since that will be checked on
	// the next play iteration (i.e. that would cause a crash)	
	return (combo == -1) ? 0 : combo;
}
