/*
    Tic-Tac-Toe 68k v1.0
    Copyright (C) 2001 John David Ratliff
    Questions or Comments: 
    	E-Mail: jdratlif@cs.indiana.edu
    	URL:	http://www.technoplaza.net 
    	
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 7/4/01; 1:39:40 PM

#include "ttt.h"
#include "gfx.h"

#include <graph.h>
#include <sprites.h>

// draw screen borders
inline void drawBorder(void) {
	DrawLine(X1,Y1,X2,Y2,A_NORMAL);
	DrawLine(X3,Y3,X4,Y4,A_NORMAL);
	DrawLine(X5,Y5,X6,Y6,A_NORMAL);
	DrawLine(X7,Y7,X8,Y8,A_NORMAL);
}

// draw a banner graphic 160x30 atop the screen
inline void drawBanner(short int banner) {
	short int loop, offset = 0;
	
	for (loop = 0; loop < 160; loop += LOGOWIDTH, offset += LOGOHEIGHT) {
		Sprite32(loop,0,LOGOHEIGHT,banners[banner]+offset,LCD_MEM,SPRT_XOR);
	}
}

// draw the game menu
inline void drawMenu(void) {
	DrawStr(13,YMIN+5,(char *)intro[PLAY],A_XOR);
	DrawStr(20,YMIN+15,(char *)intro[QUIT],A_XOR);
	DrawStr(10,YMIN+30,(char *)intro[MODE],A_XOR);
}

// draw the play mode (On-Calc/Vs. Computer/Link Play)
inline void drawMode(short int mode) {
	DrawStr(73,YMIN+30,(char *)intro[MODE+mode],A_XOR);
}

// display the copyright notice at the bottom of the screen
inline void drawCopyright(void) {
	FontSetSys(F_4x6);
	DrawStr(15,85,(char *)intro[COPYRIGHT],A_XOR);
	DrawStr(25,92,(char *)intro[URL],A_XOR);
	FontSetSys(F_6x8);
}

// draw the game board (the big #)
inline void drawBoard(void) {
	DrawLine(XMIN,YMIN+STEP,XMAX,YMIN+STEP,A_NORMAL);
	DrawLine(XMIN,YMIN+(2*STEP)+1,XMAX,YMIN+(2*STEP)+1,A_NORMAL);
	DrawLine(XMIN+STEP,YMIN,XMIN+STEP,YMAX,A_NORMAL);
	DrawLine(XMIN+(2*STEP)+1,YMIN,XMIN+(2*STEP)+1,YMAX,A_NORMAL);
}

// draws a game piece at position pos of type piece (either X or O)
inline void drawPiece(short int piece, POSITION pos) {
	Sprite16(pos.x,pos.y,PIECE_HEIGHT,pieces[piece],LCD_MEM,SPRT_XOR);
}

// draws a selector of type piece (X or O) at relative board position pos
inline void drawSelector(SQUARE *board, short int piece, short int pos) {
	if (board[pos].held == EMPTY) {
		drawPiece(piece,board[pos].pos);
	}
	
	Sprite16(board[pos].pos.x,board[pos].pos.y,PIECE_HEIGHT,pieces[EMPTY],LCD_MEM,SPRT_XOR);
}
