/*
    Tic-Tac-Toe 68k v1.0
    Copyright (C) 2001 John David Ratliff
    Questions or Comments: 
    	E-Mail: jdratlif@cs.indiana.edu
    	URL:	http://www.technoplaza.net 
    	
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Header File
// Created 6/24/2001; 12:47:50 AM

#ifndef _TTT_H
#define _TTT_H

/* Structure Definitions */

typedef struct {
	short int x, y;
} POSITION;

typedef struct {
	short int held;
	POSITION pos;
} SQUARE;

/* Constants */

#define PIECE_HEIGHT	16
#define	LOGO		3
#define LINKWAIT	200	// 20 * 10 = 10 second wait
#define	WIN_COMBOS	8
#define MAX_TRY		64
#define	TOTAL_SQUARES	9
#define	COMBO_SIZE	3

enum LogoDim	{LOGOHEIGHT = 30, LOGOWIDTH = 32};
enum KeyWait	{NOWAIT,WAIT};
enum Strings	{COPYRIGHT,URL,PLAY,QUIT,MODE};
enum Modes	{ONCALC = 1,VSCOMPUTER,LINKX,LINKO};
enum Board	{YMIN = 30, YMAX = 30+48+2, XMIN = 80-(48/2), XMAX = (80-(48/2))+48+2, STEP = 16};
enum Squares	{FIRST, LAST = 8};
enum Pieces	{X, O, EMPTY};
enum Directions	{LEFT,RIGHT,UP,DOWN};
enum Errors	{LINKERROR};
enum LinkModes	{NOTRANSLATE,TRANSLATE};
enum LinkCodes	{LINKOK,LINKUNKNOWN,LINKABORT,LINKTIMEOUT};
enum Borders	{X1 = 0, X2 = 0, X3 = 0, X4 = 159, X5 = 0, X6 = 159, X7 = 159, X8 = 159,
		 Y1 = 0, Y2 = 99, Y3 = 0, Y4 = 0, Y5 = 99, Y6 = 99, Y7 = 0, Y8 = 99};

/* External Variables */

extern short int linkmode; // TRUE if keypressed need translated, FALSE otherwise

extern const char *title;	// game title (for dialogs)
extern const char *intro[];	// intro screen strings
extern const char *errors[];	// error strings

extern unsigned short int pieces[3][16];  // piece sprites and selector
extern unsigned long int banners[4][150]; // banner logos (winners and game logo)

extern const SQUARE start[9];	// the board data

extern const short int winnings[8][3];	// winning combinations

/* Function Prototypes */

/* ttt.c */

void _main(void);

/* util.c */

inline void fatal(const char *);
short int getKey(short int);
short int translateKey(short int);
short int setupLink(short int);
short int recvCalc(short int);

/* ai.c */

short int oneToWin(SQUARE *, short int);
short int isWinnable(SQUARE *, short int);
short int findCombo(SQUARE *);
short int findMoveInCombo(SQUARE *, short int);
short int findOpenMove(SQUARE *);
short int aiPlay(SQUARE *, short int);

/* gfx.c */

inline void drawBorder(void);
inline void drawBanner(short int);
inline void drawMenu(void);
inline void drawMode(short int);
inline void drawCopyright(void);
inline void drawBoard(void);
inline void drawPiece(short int, POSITION);
inline void drawSelector(SQUARE *, short int, short int);

/* gameplay.c */

short int doIntro(void);
short int findPos(short int, short int);
void play(SQUARE *, short int, short int);
short int gameOver(SQUARE *);
void ticTacToe(short int);

#endif
